#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
from ._base import BaseHandler


class TextFileHandler(BaseHandler):

    def row_key(self, val):
        return val



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-03
"""
import os
import uuid

import shutil

from ._fileSplit import FileSplit, SubFile


class FileSort(object):

    FILE_PREFIX = '.~file_sort'

    def __init__(
            self,
            in_file_path,
            out_file_path,
            file_handler,
            block_size,
            delimiter):
        self._in = in_file_path
        self._in_dir, self._in_name = os.path.split(in_file_path)
        self._out = out_file_path
        self._handler = file_handler
        self._block_size = block_size
        self._delimiter = delimiter
        self._tmp_files = []

    def run(self):
        self._split()
        while len(self._tmp_files) > 1:
            left_file = self._tmp_files.pop(0)
            right_file = self._tmp_files.pop(0)
            self._merge_sort(
                left_file,
                right_file,
                self._gen_tmp_file())
        shutil.move(self._tmp_files.pop(0), self._out)

    def _gen_tmp_file(self):
        return os.path.join(
            self._in_dir,
            uuid.uuid4().hex
        )

    def _split(self):
        fs = FileSplit(self._in, self._block_size, self._delimiter)
        for fp_start, fp_end in fs:
            f_name = self._gen_tmp_file()
            self._save_block(fp_start, fp_end, f_name)

    def _save_block(self, start, end, out_file):
        sub_file = SubFile(self._in, start, end)
        with open(out_file, 'w') as out:
            sorted_lines = sorted(
                sub_file,
                key=lambda x: self._handler.row_key(x)
            )
            out.write(''.join(sorted_lines))
        self._tmp_files.append(out_file)

    def _merge_sort(self, left_file, right_file, out_file):
        with open(out_file, 'w') as out, \
                open(left_file) as left, \
                open(right_file) as right:
            l_line = left.readline()
            r_line = right.readline()
            l_key = self._handler.row_key(l_line)
            r_key = self._handler.row_key(r_line)
            while l_line and r_line:
                if l_key < r_key:
                    out.write(l_line)
                    for l_line in left:
                        l_key = self._handler.row_key(l_line)
                        if l_key >= r_key:
                            break
                        out.write(l_line)
                    else:
                        break
                else:
                    out.write(r_line)
                    for r_line in right:
                        r_key = self._handler.row_key(r_line)
                        if l_key < r_key:
                            break
                        out.write(r_line)
                    else:
                        break

            if l_line:
                out.write(l_line)
                for line in left:
                    out.write(line)
            else:
                out.write(r_line)
                for line in right:
                    out.write(line)
        os.remove(left_file)
        os.remove(right_file)
        self._tmp_files.append(out_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
import json

from functools import reduce

from ._textFileHandler import TextFileHandler


class JsonFileHandler(TextFileHandler):

    def __init__(self, sort_keys):
        self._keys = sort_keys

    def row_key(self, val):
        json_data = self.row_encode(val)
        key = self.row_key_plus(json_data)
        return key

    def row_key_plus(self, dic):
        key = None
        try:
            key = reduce(lambda _str, field: (
                '%s%s' % (_str, dic.get(field, ''))
            ).encode('utf8'), self._keys, '')
        except AttributeError:
            print('expect a string of dict type: %s' % dic)
        return key

    @staticmethod
    def row_encode(val):
        line = {}
        try:
            line = json.loads(val)
        except ValueError:
            print('expect a string of dict type: %s' % val)
        return line

    @staticmethod
    def row_decode(val):
        return json.dumps(val, ensure_ascii=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-10
"""


def merge(in_f, out_f, handler, merge_func=lambda dic1, dic2: dict(dic1, **dic2)):
    in_fp = open(in_f, 'r')
    out_fp = open(out_f, 'w')
    cur_line_dic = handler.row_encode(in_fp.readline())

    for line in in_fp:
        line_data = handler.row_encode(line)
        if handler.row_key_plus(line_data) != handler.row_key_plus(cur_line_dic):
            out_fp.write('%s\n' % handler.row_decode(line_data))
            cur_line_dic = line_data
            continue
        cur_line_dic = merge_func(cur_line_dic, line_data)
    out_f.write('%s\n' % handler.row_encode(cur_line_dic))
    out_f.close()


def merge_func1(dic1, dic2):
    for k, v in dic1.items():
        if v and isinstance(v, list):
            vals = dic2.get(k, [])
            if vals is None:
                vals = []
            if isinstance(v[0], dict):
                tmp_vals = [_ for _ in v if _ not in vals]
                vals += tmp_vals
            else:
                vals = list(set(vals + v))
            dic2[k] = vals
        elif v is None or (isinstance(v, (dict, list)) and len(v) == 0):
            continue
        else:
            dic2[k] = v
    return dic2



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
from ._textFileHandler import TextFileHandler
from ._jsonFileHandler import JsonFileHandler



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
import os

import yaml


class Config(object):

    def __init__(self):
        default = yaml.load(open(os.path.join(
            os.path.dirname(__file__), 'default_configs.yaml'
        ), 'r'))
        for key, value in default:
            setattr(self, key, value)


CONFIG = Config()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-08
"""


def get_CPU_core_num():
    from multiprocessing import cpu_count

    return cpu_count()


def get_file_size(file_path):
    from os.path import getsize

    return getsize(file_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
from ._fileSort import FileSort
from .handler import TextFileHandler
from .handler import JsonFileHandler
from ._fileMerge import merge, merge_func1


__all__ = ('JsonFileHandler', 'TextFileHandler', 'sort_json_file', 'sort_file')


def sort_file(
        input_file,
        output_file,
        file_handler=None,
        row_delimiter='\n',
        block_size=None
    ):
    """

    :param input_file:
    :param output_file:
    :param file_handler:
    :param row_delimiter: 行分割符
    :param block_size: 分块大小
    :return:
    """
    from ._utils import get_CPU_core_num

    if block_size is None:
        block_size = 1024*1024*32  # 32MB
    if file_handler is None:
        file_handler = TextFileHandler()

    FileSort(
        input_file, output_file, file_handler, block_size, row_delimiter).run()


def sort_json_file(
        input_file,
        output_file,
        sort_keys,
        row_delimiter='\n',
        block_size=None,
    ):

    j_handler = JsonFileHandler(sort_keys)
    sort_file(input_file, output_file, j_handler, row_delimiter, block_size)


def merge_json_file(
        input_file,
        output_file,
        merge_keys,
        row_delimiter='\n',
        block_size=None
    ):

    tmp_sort_file = '%s.sorted' % output_file
    j_handler = JsonFileHandler(merge_keys)
    sort_file(input_file, output_file, j_handler, row_delimiter, block_size)
    merge(tmp_sort_file, output_file, j_handler, merge_func1)



from setuptools import setup, find_packages


setup(
    name='PyTextSort',
    version='1.0.7',
    description='custom rules to sort a text file',
    author='djj',
    url='https://github.com/djjjj/py-text-sort',
    author_email='510620510@qq.com',
    packages=find_packages(),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    hack by djj -_,- | good luck!
    date: 2018-03-02
"""
import sys

from ._utils import get_file_size


if sys.version_info < (3, 0):
    str = lambda x, y: x.encode(y, 'ignore')
    handle_delimiter = lambda x: x
else:
    handle_delimiter = lambda x: ord(x)


class SubFile(object):
    def __init__(self, file_path, start, end):
        fp = open(file_path, 'rb')
        fp.seek(start)
        self._fp = fp
        self._len = end - start

    def __iter__(self):
        line = self._fp.readline()
        self._len -= len(line)
        while self._len >= 0 and line:
            yield str(line, 'utf8')
            line = self._fp.readline()
            self._len -= len(line)
        self._fp.close()


class FileSplit(object):

    def __init__(self, in_file_path, block_size, delimiter):
        self._ret = []
        self._fp = open(in_file_path, 'rb')
        self._delimiter = handle_delimiter(delimiter)
        self._size = get_file_size(in_file_path)
        self._split_num = self._size // block_size + 1
        assert self._split_num <= 1024

    def __iter__(self):
        step = self._size // self._split_num
        end = 0
        for i in range(self._split_num):
            start = end
            end = start + step
            self._fp.seek(end)
            # find the char '\n' or other delimiter
            buf = self._fp.read(2048)
            flag = False
            while buf:
                for c in buf:
                    end += 1
                    if c == self._delimiter:
                        flag = True
                        break
                if flag:
                    break
                buf = self._fp.read(2048)
            yield start, end
        self._fp.close()
